#游戏引擎：控制游戏流程
import models,pygame,random,sys,traceback
from pygame.locals import *
# import os
# CMD = r'D:\ImageMagick-7.0.8-Q16-HDRI\convert.exe'   #ImageMagick安装目录下convert.exe所在目录
# SOURCE_PATH = r'E:\作业练习\游戏开发\bg'                          #png图片所在目录
#
# def doStrip(path):
#     data = {}
#     print(path)v
#     for root, dir
# s, files in os.walk(path):
#         for file in files:
#             name = file.lower()
#             if name.find('.png') != -1:
#                 path = os.path.join(root, file)
#                 os.system('"{0}" {1} -strip {1}'.format(CMD, path, path))
# doStrip(SOURCE_PA
# 添加背景音乐
# pygame.mixer.init()
# pygame.mixer.music.load("毛毛 - 奇迹再现.mp3")
# pygame.mixer.music.set_volume(0.5)  #设置音量



#对要用到的颜色进行宏定义
color_black = (0,0,0)
color_green = (0,255,0)
color_red = (255,0,0)
color_white = (255,255,255)
color_blue = (0,0,255)

class Game_Engine():
    def __init__(self):
        #引擎类型中，初始化服务对象

        #定义背景精灵
        self.bg1 = models.BackgroundSprite()
        self.bg2 = models.BackgroundSprite(next = True)
        #定义英雄飞机对象
        self.hero = models.HeroSprite()
        #定义英雄飞机对象总血量
        self.energy = self.hero.energy

        #定义补给一（子弹强化）对象
        self.supply1 = models.Supply1()
        #定义补给二（神圣护盾）对象
        self.supply2 = models.Supply2()
        #定义一个补给一（子弹强化）的精灵组对象
        self.supply_first = pygame.sprite.Group(self.supply1)
        #定义一个补给二（神圣护盾）的精灵组对象
        self.supply_sencond = pygame.sprite.Group(self.supply2)
        #定义一个敌机对象
        self.enemy = models.EnemySprite()

        #定义一个敌机boss对象
        self.enemy_boss = models.Enemy_Boss()

        #定义初始化精灵组对象
        self.resources = pygame.sprite.Group(self.bg1,self.bg2,self.hero)

        #再创建一个英雄飞机精灵组，，用来实现英雄飞机与敌方boss飞机之间的碰撞
        self.hero_boss = pygame.sprite.Group(self.hero)


        # 定义一个敌机的精灵组对象
        self.enemys = pygame.sprite.Group()

        #定义一个boss敌机的精灵组对象
        self.enemys_boss = pygame.sprite.Group()

        #定义一个敌机被摧毁的精灵组对象
        self.destroy_images =  pygame.sprite.Group()
        #同时创建一个字典用以保存被摧毁的敌机的数据
        self.destroy_images_dict = {}

    def start(self):
        # 初始化所有模块
        pygame.init()
        clock = pygame.time.Clock()
        #设定敌机出现的时间
        pygame.time.set_timer(models.ENEMY_CREATE,2000)
        #设定boss敌机出现的时间（在游戏进行一分钟时出现）
        pygame.time.set_timer(models.ENEMY_CREATE +1,20000)
        self.create_scene()
        #游戏场景循环
        while True:
            # pygame.mixer.music.play(-1)  # 循环播放
            #定义时钟刷新帧：每秒让循环运行多少次！
            clock.tick(24)
            self.update_scene() #创建游戏场景并渲染精灵组
            self.check_event() #监听所有的事件&获取当前用户在键盘上操作的按键
            self.check_collide() #碰撞检测
            self.blood_groove()  #对英雄飞机绘制血槽
            self.boss_blood_groove() #对敌方boss飞机绘制血槽
            self.create_enetgy_list()  #绘制能量表
    def blood_groove(self):
    #对英雄飞机生命值绘制血槽
        pygame.draw.line(self.screen, color_blue, (self.hero.rect.left, self.hero.rect.top - 5),
                        (self.hero.rect.right, self.hero.rect.top - 5), 15)
        energy_remain = self.hero.blood_volume / 30  # 英雄飞机当前血量除以英雄飞机总血量
        if energy_remain > 0.4:  # 如果血量大于百分之二十则为绿色，否则为红色
            energy_color = color_green
        else:
            energy_color = color_red
        pygame.draw.line(self.screen, energy_color,
        (self.hero.rect.left, self.hero.rect.top - 5),
        (self.hero.rect.left + self.hero.rect.width * energy_remain, self.hero.rect.top - 5),
        2)
        #更新血槽图片
        pygame.display.update()

    def boss_blood_groove(self):
        #对敌方boss飞机生命值绘制血槽
        pygame.draw.line(self.screen,color_blue,(self.enemy_boss.rect.left,self.enemy_boss.rect.top -5),
                         (self.enemy_boss.rect.right,self.enemy_boss.rect.top -5),15)
        energy_remain_boss = self.enemy_boss.boss_blood_volume / 200 #敌方boss飞机当前血量除以敌方boss飞机总血量
        if energy_remain_boss > 0.4:  #如果血量大于百分之四十则为绿色，否则为红色
            energy_color = color_green
        else:
            energy_color = color_red
        pygame.draw.line(self.screen,energy_color,(self.enemy_boss.rect.left,self.enemy_boss.rect.top -5),
                         (self.enemy_boss.rect.left + self.enemy_boss.rect.width * energy_remain_boss,self.enemy_boss.rect.top-5),2)
        #更新血槽图片
        pygame.display.update()

    def create_scene(self):
        #创建游戏场景
        self.screen = pygame.display.set_mode(models.SCREEN_SIZE)
        # 窗口名称的设置
        pygame.display.set_caption("塑料版飞机大战")

    def create_enetgy_list(self):
        # 使用系统字体
        self.fonts = pygame.font.SysFont("fangsong", 20, True)
        # 加粗
        self.fonts.set_bold(True)
        # 斜体
        self.fonts.set_italic(True)
        #创建一个能量表，用以积蓄能量
        self.screen.blit(self.fonts.render("当前能量：%s" % self.hero.energy, True, color_blue, color_red),[10,10])
        #刷新游戏场景
        pygame.display.update()

    def update_scene(self):
        #新增英雄飞机精灵组渲染
        self.hero_boss.update()
        self.hero_boss.draw(self.screen)

        #精灵组渲染
        self.resources.update()
        self.resources.draw(self.screen)

        #英雄子弹精灵组渲染
        self.hero.bullets.update()
        self.hero.bullets.draw(self.screen)
        #英雄加强版子弹精灵组渲染
        self.hero.strengthen_bullets.update()
        self.hero.strengthen_bullets.draw(self.screen)

        #普通敌机子弹精灵组渲染
        self.enemy.destroy_bullets.update()
        self.enemy.destroy_bullets.draw(self.screen)
        #boss敌机子弹精灵组渲染
        self.enemy_boss.enemy_boss_bullets.update()
        self.enemy_boss.enemy_boss_bullets.draw(self.screen)

        #渲染敌机精灵组中的所有飞机
        self.enemys.update()
        self.enemys.draw(self.screen)

        #渲染boss敌机精灵组中的所有飞机
        self.enemys_boss.update()
        self.enemys_boss.draw(self.screen)

        #补给精灵组一渲染
        self.supply_first.update()
        self.supply_first.draw(self.screen)
        #补给精灵组二渲染
        self.supply_sencond.update()
        self.supply_sencond.draw(self.screen)

        # 更新游戏场景
        pygame.display.update()

    def check_collide(self):
        #碰撞检测
        #碰撞检测：子弹与敌方飞机之间的碰撞
        self.destroy_images_dict  =  pygame.sprite.groupcollide(self.hero.bullets,self.enemys,True,True)
        #将被毁灭的敌机添加到敌机被摧毁的精灵组中
        self.destroy_images.add(self.destroy_images_dict)
        for destroy_image in self.destroy_images:

            #调整帧率，实现慢速爆炸的效果
            clock = pygame.time.Clock()
            clock.tick(20)

            self.destroy(destroy_image)#将敌机返回到敌机被摧毁的精灵组中，调用函数
            print("爆炸>>>>>>>")
            self.destroy_images.remove(destroy_image)
        while len(self.destroy_images_dict) > 0:
            self.hero.energy += 5
            break

        #英雄加强版子弹与敌机之间的碰撞
        self.destroy_images_dict = pygame.sprite.groupcollide(self.hero.strengthen_bullets, self.enemys, True, True)
        # 将被毁灭的敌机添加到敌机被摧毁的精灵组中
        self.destroy_images.add(self.destroy_images_dict)
        for destroy_image in self.destroy_images:
            # 调整帧率，实现慢速爆炸的效果
            clock = pygame.time.Clock()
            clock.tick(20)

            self.destroy(destroy_image)  # 将敌机返回到敌机被摧毁的精灵组中，调用函数
            print("爆炸>>>>>>>")
            self.destroy_images.remove(destroy_image)
        while len(self.destroy_images_dict) > 0:
            self.hero.energy += 5
            break

        # 调整帧率，以实现英雄飞机与敌方boss碰撞时英雄飞机慢速减血的效果
        clock = pygame.time.Clock()

        #碰撞检测：英雄飞机子弹与敌方boss飞机之间的碰撞
        u = pygame.sprite.groupcollide(self.hero.bullets,self.enemys_boss,True,False)
        while len(u) > 0:

            clock.tick(24)

            self.enemy_boss.boss_blood_volume -= 1
            if self.enemy_boss.boss_blood_volume > 0: #判断敌方boss飞机血量
                print("敌方boss剩余血量" , self.enemy_boss.boss_blood_volume)
                break
            elif self.enemy_boss.boss_blood_volume == 0:
                self.enemy_boss.kill()
                pygame.quit()
                exit()

        #碰撞检测：英雄飞机与敌方boss飞机之间的碰撞
        e = pygame.sprite.groupcollide(self.hero_boss,self.enemys_boss,False,False)
        while len(e) > 0: #判断英雄飞机血量
            self.hero.blood_volume -= 1
            if self.hero.blood_volume > 0:  #判断英雄飞机血量
                print("剩余生命：", self.hero.blood_volume)
                break
            elif self.hero.blood_volume == 0:
                self.hero.kill()
                pygame.quit()
                exit()

        #碰撞检测： 英雄飞机与敌方boss子弹之间的碰撞
        b = pygame.sprite.spritecollide(self.hero,self.enemy_boss.enemy_boss_bullets,True)
        while len(b) > 0:
            self.hero.blood_volume -= 1
            if self.hero.blood_volume > 0:  #判断英雄飞机血量
                print("剩余生命：", self.hero.blood_volume)
                break
            elif self.hero.blood_volume == 0:
                # 判断英雄飞机血量
                self.hero.kill()
                pygame.quit()
                exit()
        #英雄飞机与补给物品一（子弹强化）之间的碰撞
        x = pygame.sprite.spritecollide(self.hero,self.supply_first,True)
        if len(x) > 0:
            self.hero.strengthen_fire()
        #英雄飞机与补给物品二（神圣护盾）之间的碰撞
        z = pygame.sprite.spritecollide(self.hero,self.supply_sencond,True)
        if len(z) > 0:
            pass#英雄飞机得到一个护盾，在六秒钟内达到无敌的效果

        #碰撞检测：英雄飞机和敌方飞机之间的碰撞
        e = pygame.sprite.spritecollide(self.hero,self.enemys,True)
        while len(e) >0:
            self.hero.blood_volume -= 1
            if self.hero.blood_volume > 0:  #判断英雄飞机血量
                print("剩余生命：",self.hero.blood_volume)
                break
            elif self.hero.blood_volume == 0:
                self.hero.kill()
                pygame.quit()
                exit()

        #碰撞检测：英雄飞机和敌机子弹之间的碰撞
        c = pygame.sprite.spritecollide(self.hero,self.enemy.destroy_bullets,True)
        while len(c) >0:
            self.hero.blood_volume -= 1
            if self.hero.blood_volume > 0:  #判断英雄飞机血量
                print("剩余生命：",self.hero.blood_volume)
                break
            elif self.hero.blood_volume == 0:
                self.hero.kill()
                pygame.quit()
                exit()

    def destroy(self,enemy):
        # 敌机被摧毁
        print("敌机销毁")
        # 添加飞机被摧毁的过程图
        for i in range(2,5):
            clock = pygame.time.Clock()
            clock.tick(35)
            destroy_image = pygame.image.load("./bg/destroy_images" + str(i) + ".png")
            self.screen.blit(destroy_image,enemy.rect)
            pygame.display.update()
            print(destroy_image)

    #让pygame完全控制鼠标
    sprite_speed = 300
    sprite_rotation = 0
    sprite_rotation_speed = 360


    def check_event(self):
        #监听所有的事件
        event_list = pygame.event.get()
        if len(event_list) > 0:
            print(event_list)
            for event in event_list:
                print(event.type,pygame.KEYDOWN,pygame.K_LEFT)
                #如果当前的事件，是QUIT事件
                if event.type == pygame.QUIT:
                    #卸载所有pygame资源 ，退出程序
                    pygame.quit()
                    exit()

                elif event.type == models.ENEMY_CREATE:
                    print("创建一架敌方飞机....")
                    self.enemy = models.EnemySprite()
                    #添加到敌方飞机精灵组中
                    self.enemys.add(self.enemy)
                elif event.type == models.ENEMY_CREATES:
                    print("创建一架敌方boss飞机....")
                    self.enemy_boss = models.Enemy_Boss()
                    #添加到敌方boss飞机精灵组中
                    self.enemys_boss.add(self.enemy_boss)


             #添加敌机子弹到敌机对象中
            self.enemy.destroy_fire()
        # # 自动发射子弹的实现
        # self.hero.fire()
        # print("发射子弹》》》", self.hero.bullets)

        # #使用鼠标来操控飞机
        # pos = pygame.mouse.get_pos()
        # self.hero.rect.x = pos[0] - self.he
        # self.hero.rect.y = pos[1] - self.he
        # pygame.mouse.set_visible(False)

        # pressed_keys = pygame.key.get_pressed()
        # #定位鼠标的x.y坐标&获取当前用户鼠标的操作
        # pressed_mouse = pygame.mouse.get_pressed()
        # #通过移动偏移量计算移动
        # rotation_direction = pygame.mouse.get_rel()[0] /2.0
        # if pressed_mouse[0]:
        #     print("向左移动<<<<<<<<<<<<")
        #     self.hero.rect.x -= 15
        # elif pressed_mouse[2]:
        #     print("向右移动>>>>>>>>>>>>")
        #     self.hero.rect.x += 15
        # elif pressed_mouse[1]:
        #     print("向上移动^^^^^^^^^^^")
        #     self.hero.rect.y -= 15
        # elif pressed_mouse[1]:
        #     print("向下移动vvvvvvvvvvv")
        #     self.hero.rect.y += 15

        #获取当前用户键盘上被操作的按键
        key_down = pygame.key.get_pressed()

        if key_down[pygame.K_LEFT]:
            print("向左移动<<<<<<<<<<<<")
            self.hero.rect.x -= 15
        elif key_down[pygame.K_RIGHT]:
            print("向右移动>>>>>>>>>>>>")
            self.hero.rect.x += 15
        elif key_down[pygame.K_UP]:
            print("向上移动^^^^^^^^^^^")
            self.hero.rect.y -= 15
        elif key_down[pygame.K_DOWN]:
            print("向下移动vvvvvvvvv")
            self.hero.rect.y += 15

        if key_down[pygame.K_SPACE]:
            self.hero.fire()
            print("发射子弹》》》", self.hero.bullets )
        if key_down[pygame.K_q]:
            self.hero.strengthen_fire()
            print("发射强化版子弹》》》", self.hero.strengthen_bullets)


#所有游戏精灵类型存放的文件

import pygame,random,traceback
from pygame.locals import *

# #定义需要的常量
SCREEN_SIZE = (512,768)
SCREEN_RECT = pygame.Rect(0,0,*SCREEN_SIZE) #定义游戏场景的大小

# #自定义一个事件
ENEMY_CREATE = pygame.USEREVENT

#自定义一个事件，用来定义敌方boss的出现时间
ENEMY_CREATES = pygame.USEREVENT +1

#时间声明
clock = pygame.time.Clock()


class GameSprite(pygame.sprite.Sprite):
    #游戏精灵对象：用于表示游戏中的各种元素
    def __init__(self,image_path,speed = 1):
        #调用父类初始化数据
        super().__init__()
        self.image = pygame.image.load(image_path)
        self.rect = self.image.get_rect()
        self.speed = speed

    def update(self):
        #默认运动更新方法
        self.rect.y += self.speed

class BackgroundSprite(GameSprite):
    def __init__(self,next = False):
        super().__init__("./bg/img_bg_level_2.jpg",speed = 5)
        if next:
            self.rect.y = - SCREEN_SIZE[1]
    def update(self):
        #调用父类的方法，执行运动
        super().update()
        #子类中判断边界
        if self.rect.y > SCREEN_SIZE[1]:
            self.rect.y = -SCREEN_SIZE[1]
class HeroSprite(GameSprite):
    #英雄飞机精灵对象
    #添加energy成员变量用来表示英雄飞机血量
    # energy = 5
    def __init__(self):
        #初始化英雄飞机的图片、速度
        super().__init__("./bg/hero.png",speed = 0)
        #初始化英雄飞机的位置
        self.rect.centerx = SCREEN_RECT.centerx
        self.rect.y = SCREEN_RECT.centery + 200
        #定义并初始化英雄飞机的血量
        self.blood_volume = 30
        #定义初始化定义英雄飞机的能量
        self.energy = 0
        #创建一个英雄飞机子弹精灵组
        self.bullets = pygame.sprite.Group()

        #创建一个英雄飞机加强版子弹精灵组
        self.strengthen_bullets = pygame.sprite.Group()

    def update(self):
        #水平边界判断
        if self.rect.x <= 0:
            self.rect.x = 0
        elif self.rect.x >= SCREEN_RECT.width - self.rect.width:
            self.rect.x = SCREEN_RECT.width - self.rect.width
        #垂直边界判断
        if self.rect.y <= 0:
            self.rect.y = 0
        elif self.rect.y >= SCREEN_RECT.height - self.rect.height:
            self.rect.y = SCREEN_RECT.height - self.rect.height

    def fire(self):
        #飞机攻击 &创建一个子弹对象
        bullet = BulletSprite(self.rect.centerx-63,self.rect.y)
        # 添加到精灵组对象
        self.bullets.add(bullet)
    def strengthen_fire(self):
        #飞机攻击  &创建一个加强版子弹对象
        reinforced_bullet = Reinforced_Bullet(self.rect.centerx-10,self.rect.y)
        #将加强版英雄子弹添加到加强版子弹精灵组对象
        self.strengthen_bullets.add(reinforced_bullet)

class BulletSprite(GameSprite):

    #子弹精灵
    def __init__(self,x,y):
        super().__init__("./bg/pic2.png",speed = -8)
        self.rect.x = x
        self.rect.y = y
        #self.speed = speed
    def update(self):
        #调用父类的方法进行操作
        super().update()
        #边界判断
        if self.rect.y <= -self.rect.height:
            #子弹从精灵组中删除
            self.kill()
    def __del__(self):
        print("子弹对象已经销毁")


class Reinforced_Bullet(GameSprite):
    #英雄加强版子弹精灵
    def __init__(self,x,y):
        super().__init__("./bg/bullet1.png",speed = -16)
        self.rect.x = x-180
        self.rect.y = y
    def update(self):
        #调用分类的方法进行操作
        super().update()
        #边界判断
        if self.rect.y <= -self.rect.height:
            #子弹从精灵组中删除
            self.kill()
    def __del__(self):
        print("加强版子弹已经销毁")


class Destroy_Bullet_Sprite(GameSprite):
    #普通敌机子弹精灵
    def __init__(self,x,y):
        super().__init__("./bg/destroy_bullets.png", speed = 16)
        self.rect.x = x
        self.rect.y = y
    def update(self):
        #调用父类的方法进行操作
        super().update()
        #边界判断
        if self.rect.y <= -SCREEN_RECT.height:
            #子弹从精灵组中删除
            self.kill()
    def __del__(self):
        print("敌机子弹已销毁")

class Destroy_Boss_Bullet(GameSprite):
    #boss敌机子弹精灵
    def __init__(self,x,y):
        super().__init__("./bg/destroy_bullets.png", speed = 14)
        self.rect.x = x
        self.rect.y = y
    def update(self):
        #调用父类的方法进行操作
        super().update()
        #边界判断
        if self.rect.y <= -SCREEN_RECT.height:
            #子弹从精灵组中删除
            self.kill()
    def __del__(self):
        print("敌机子弹已销毁")

class EnemySprite(GameSprite):
    #敌方飞机
    def __init__(self):
        #初始化敌方飞机的数据：图片，速度
        super().__init__("./bg/png1.png",speed = random.randint(3,5))
        #初始化敌方飞机的位置
        self.rect.x = random.randint(0,SCREEN_RECT.width - self.rect.width)
        self.rect.y = -self.rect.height
        #创建一个敌方飞机子弹精灵组
        self.destroy_bullets = pygame.sprite.Group()

    def destroy_fire(self):
        #敌机进行攻击 & 创建一个子弹对象
        destroy_bullet = Destroy_Bullet_Sprite(self.rect.centerx -31,self.rect.y)
        #添加到敌机子弹精灵组对象
        self.destroy_bullets.add(destroy_bullet)

    def update(self):
        #调用父类的方法直接运动
        super().update()
        #边界判断
        if self.rect.y > SCREEN_RECT.height:
            #飞机一旦超出屏幕，销毁！
            self.kill()

class Enemy_Boss(GameSprite):
    #敌方boss飞机
    def __init__(self):
        #初始化敌方boss飞机的数据： 图片，速度
        super().__init__("./bg/boss.png",speed = 5)

        #初始化敌方boss飞机的位置
        self.rect.x = random.randint(0,SCREEN_RECT.width - self.rect.width)
        self.rect.y =  - self.rect.height


        # 定义并初始化敌方boss飞机的血量
        self.boss_blood_volume = 200

        #创建一个敌方boss飞机子弹精灵组
        self.enemy_boss_bullets = pygame.sprite.Group()
    def destroy_boss_fire(self):
        #敌方boss进行攻击 & 创建一个boss子弹对象
        destroy_boss_bullets = Destroy_Boss_Bullet(self.rect.centerx -31,self.rect.y)
        #添加到敌机boss子弹精灵组对象
        self.enemy_boss_bullets.add(destroy_boss_bullets)

    def update(self):
        #重写父类方法进行运动
        super().update()
        #边界判断
        self.rect.x -= self.speed
        if self.rect.x <= 0:
            self.rect.x = 0

        elif self.rect.x >= SCREEN_RECT.width - self.rect.width:
            self.rect.x = SCREEN_RECT.width - self.rect.width

        if self.rect.y <= 0:
            self.rect.y = 0
            self.speed = -self.speed
        elif self.rect.y >= SCREEN_RECT.centery - self.rect.height:
            self.rect.y = SCREEN_RECT.centery - self.rect.height


class Supply1(GameSprite):
    def __init__(self):
        #初始化补给物品一（子弹加强）的数据：：图片，速度
        super().__init__("./bg/purple.png" ,speed = random.randint(2,5))
        #初始化补给物品一（子弹加强）的落下位置
        self.rect.x = random.randint(0,SCREEN_RECT.width - self.rect.width)
        self.rect.y = -self.rect.height

    def update(self):
        #调用父类的方法直接运动
        super().update()
        #边界判断
        if self.rect.y > SCREEN_RECT.height:
            #补给一旦超出屏幕，错过也就错过了，所谓爱情，不过如此！
            self.kill()
class Supply2(GameSprite):
    def __init__(self):
        #初始化补给物品二(神圣护盾)的数据，：图片，速度
        super().__init__("./bg/shield_blue.png" , speed = random.randint(2,5))
        #初始化补给物品二（神圣护盾）的落下位置
        self.rect.x = random.randint(0,SCREEN_RECT.width - self.rect.width)
        self.rect.y = -self.rect.height
    def update(self):
        #调用分类的方法直接运动
        super().update()
        #边界判断
        if self.rect.y > SCREEN_RECT.height:
            #补给一旦超出屏幕，错过也就错过了，所谓爱情，不过如此！
            self.kill()







#引入构建包信息的模块
from distutils.core import setup
#定义发布的包文件的信息
setup(
    name = "Aircraft_war", #发布的包文件名称
    version = "1.0", #发布的包的版本序号
    description = "我的测试包" , #发布包的描述信息
    author = "Ning_Caichen" ,#发布包的作者信息
    author_email = "975973347@qq.com" , #作者的联系邮箱信息
    py_modules = ['__init__','main','models','game_engine'] #发布的保重的模块文件列表
)